#include "restclient.h"

#include <iomanip>
#include <Log.h>

#include <QNetworkRequest>
#include <QNetworkReply>
#include <QUrl>

RestClient::RestClient(QObject* parent, const QString& base) : QObject(parent), baseurl(base), net( new QNetworkAccessManager(this) ) {
	net->setRedirectPolicy(QNetworkRequest::RedirectPolicy::NoLessSafeRedirectPolicy);
}

RestClient::~RestClient() {
}

RestClient::WebException::WebException(int c) : curlresponse(c) {
}

QByteArray RestClient::requestRaw(const QString& url, Method m, const QString& data) {
	QString requrl = baseurl + url;

	QNetworkRequest req( QUrl{requrl} );

	if(!basicauth.isEmpty()) {
		req.setRawHeader("Authorization", "Basic " + QByteArray(basicauth.toLocal8Bit()).toBase64());
	}

	if (disableCertCheck) {
		QSslConfiguration ssl = req.sslConfiguration();
		ssl.setPeerVerifyMode(QSslSocket::PeerVerifyMode::VerifyNone);
		req.setSslConfiguration(ssl);
	}

	req.setRawHeader("Accept", "application/json");

	QNetworkReply* repl = nullptr;
	switch (m) {
	default:
	case Method::GET:
		repl = net->get(req); break;
	case Method::POST:
		req.setHeader(QNetworkRequest::KnownHeaders::ContentTypeHeader, "application/json");
		repl = net->post(req, data.toLocal8Bit());
		break;
	case Method::PUT:
		req.setHeader(QNetworkRequest::KnownHeaders::ContentTypeHeader, "application/json");
		repl = net->put(req, data.toLocal8Bit());
		break;
	case Method::PATCH:
		req.setHeader(QNetworkRequest::KnownHeaders::ContentTypeHeader, "application/json");
		repl = net->sendCustomRequest(req, "PATCH", data.toLocal8Bit());
		break;
	case Method::DELETE:
		req.setHeader(QNetworkRequest::KnownHeaders::ContentTypeHeader, "application/json");
		repl = net->sendCustomRequest(req, "DELETE", data.toLocal8Bit());
		break;
	}

	// set callback data
	if(!repl->waitForReadyRead(5000)) {
		qCritical() << "request timed out";
		return {};
	}

	QByteArray arr = repl->readAll();

	unsigned int res = repl->attribute(QNetworkRequest::Attribute::HttpStatusCodeAttribute).toUInt();

	// Check for errors
	if (res != 200) {
		qWarning() << "API request failed: " << baseurl << " " << url << " -> " << res;
		if(res == 0) {
			throw WebException(res);
		}
		return {};
	}
	return arr;
}

QJsonDocument RestClient::request(const QString& url, Method m, const QString& data) {
	QByteArray arr = requestRaw(url, m, data);
	if (arr.isEmpty()) return {};

	QJsonParseError err;
	QJsonDocument parsed = QJsonDocument::fromJson(arr, &err);
	if (parsed.isNull() || err.error != QJsonParseError::NoError) {
		qCritical() << "API Json parse error " << err.errorString() << " offset: " << err.offset;
		return {};
	}

	return parsed;
}

void RestClient::enableDebugging(bool enabled) {
	debugging = enabled;
}

QString RestClient::escape(const QString& in) const {
	return QUrl::toPercentEncoding(in);
}
